import Editor from '@modules/editor';
import { LineSegments2 } from 'three/examples/jsm/lines/LineSegments2';
import { Object3D } from 'three';

const EVENTS = {
  READY: 'ready',
  RESIZE: 'resize',
  REMOVE: 'remove',
  LOAD_START: 'load_start',
  LOAD: 'load',
  LOAD_FAIL: 'load_fail',
  LINE_DISTANCE: 'line_distance',
  RENDER: 'render',
  OBJECT_CLICK: 'object_click',
} as const;

interface ObjectClickEvent {
  type: string;
  target: Object3D;
}

interface ReadyEvent {
  type: string;
  target: Editor;
}

interface ResizeEvent {
  type: string;
  target: Editor;
}

interface LoadStartEvent {
  type: string;
  target: Editor;
  loadStartData?: any;
}

interface LoadEvent {
  type: string;
  target: Editor;
  id: number | string;
  loadData?: any;
}

interface LoadFailEvent {
  type: string;
  target: Editor;
  msg?: any;
}

interface RemoveEvent {
  type: string;
  target: Editor;
}

interface LineDistanceEvent {
  type: string;
  target: LineSegments2;
  setLineDistance: (distance: number | 'delete') => void;
}

interface RenderEvent {
  type: string;
  target: Editor;
}
export interface Events {
  [EVENTS.READY]: ReadyEvent;
  [EVENTS.LOAD_START]: LoadStartEvent;
  [EVENTS.LOAD]: LoadEvent;
  [EVENTS.RESIZE]: ResizeEvent;
  [EVENTS.REMOVE]: RemoveEvent;
  [EVENTS.LOAD_FAIL]: LoadFailEvent;
  [EVENTS.LINE_DISTANCE]: LineDistanceEvent;
  [EVENTS.RENDER]: RenderEvent;
  [EVENTS.OBJECT_CLICK]: ObjectClickEvent;
}
